@@ -126,11 +126,12 @@ day_selected (GtkCalendar *calendar, GnomeDateEdit *gde)
 	char buffer [256];
 	guint year, month, day;
 	struct tm mtm = {0};
+	char *str_utf8;
 
 	gtk_calendar_get_date (calendar, &year, &month, &day);
 
 	mtm.tm_mday = day;
-	mtm.tm_mon = month - 1;
+	mtm.tm_mon = month;
 	if (year > 1900)
 		mtm.tm_year = year - 1900;
 	else
@@ -139,10 +140,14 @@ day_selected (GtkCalendar *calendar, GnomeDateEdit *gde)
 	if (strftime (buffer, sizeof (buffer),
 		      strftime_date_format, &mtm) == 0)
 		strcpy (buffer, "???");
+	buffer[sizeof(buffer)-1] = '\0';
 
 	/* FIXME: what about set time */
 
-	gtk_entry_set_text (GTK_ENTRY (gde->_priv->date_entry), buffer);
+	str_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+	gtk_entry_set_text (GTK_ENTRY (gde->_priv->date_entry),
+			    str_utf8 ? str_utf8 : "");
+	g_free (str_utf8);
 	gtk_signal_emit (GTK_OBJECT (gde), date_edit_signals [DATE_CHANGED]);
 }
 
@@ -219,7 +224,7 @@ position_popup (GnomeDateEdit *gde)
 	gtk_widget_size_request (gde->_priv->cal_popup, &req);
 
 	gdk_window_get_origin (gde->_priv->date_button->window, &x, &y);
-	gdk_window_get_size (gde->_priv->date_button->window, &bwidth, &bheight);
+	gdk_drawable_get_size (gde->_priv->date_button->window, &bwidth, &bheight);
 
 	x += bwidth - req.width;
 	y += bheight;
@@ -238,14 +243,17 @@ select_clicked (GtkWidget *widget, GnomeDateEdit *gde)
 {
 	const char *str;
 	GDate *date;
+	int month;
 
 	str = gtk_entry_get_text (GTK_ENTRY (gde->_priv->date_entry));
 
 	date = g_date_new ();
 	g_date_set_parse (date, str);
-
+        /* GtkCalendar expects month to be in 0-11 range (inclusive) */
+	month = g_date_get_month (date) - 1;
+	
 	gtk_calendar_select_month (GTK_CALENDAR (gde->_priv->calendar),
-				   g_date_get_month (date),
+				   CLAMP (month, 0, 11),
 				   g_date_get_year (date));
         gtk_calendar_select_day (GTK_CALENDAR (gde->_priv->calendar),
 				 g_date_get_day (date));
@@ -323,7 +331,6 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 		return;
 
 	menu = gtk_menu_new ();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (gde->_priv->time_popup), menu);
 
 	time (&current_time);
 	mtm = localtime (&current_time);
@@ -332,6 +339,7 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 		GtkWidget *item, *submenu;
 		hour_info_t *hit;
 		char buffer [40];
+		char *str_utf8;
 
 		mtm->tm_hour = i;
 		mtm->tm_min  = 0;
@@ -345,7 +353,10 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 				      "%I:00 %p ", mtm) == 0)
 				strcpy (buffer, "???");
 		}
-		item = gtk_menu_item_new_with_label (buffer);
+		buffer[sizeof(buffer)-1] = '\0';
+		str_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+		item = gtk_menu_item_new_with_label (str_utf8 ? str_utf8 : "");
+		g_free (str_utf8);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 #if 0
 		hit = g_new (hour_info_t, 1);
@@ -377,8 +388,11 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 					      "%I:%M %p", mtm) == 0)
 					strcpy (buffer, "???");
 			}
+			buffer[sizeof(buffer)-1] = '\0';
+			str_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+			mins = gtk_menu_item_new_with_label (str_utf8 ? str_utf8 : "");
+			g_free (str_utf8);
 
-			mins = gtk_menu_item_new_with_label (buffer);
 			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mins);
 
 			hit = g_new (hour_info_t, 1);
@@ -395,6 +409,8 @@ fill_time_popup (GtkWidget *widget, GnomeDateEdit *gde)
 			gtk_widget_show (mins);
 		}
 	}
+	/* work around a GtkOptionMenu bug #66969 */
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (gde->_priv->time_popup), menu);
 }
 
 static void
@@ -418,7 +434,7 @@ gnome_date_edit_class_init (GnomeDateEditClass *class)
 					 PROP_TIME,
 					 g_param_spec_ulong ("time",
 							     _("Time"),
-							     _("The time currently"
+							     _("The time currently "
 							       "selected"),
 							     0, G_MAXULONG,
 							     0,
@@ -629,7 +645,8 @@ void
 gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 {
 	struct tm *mytm;
-	char buffer [40];
+	char buffer [256];
+	char *str_utf8;
 
 	g_return_if_fail(gde != NULL);
 
@@ -642,7 +659,11 @@ gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 	/* Set the date */
 	if (strftime (buffer, sizeof (buffer), strftime_date_format, mytm) == 0)
 		strcpy (buffer, "???");
-	gtk_entry_set_text (GTK_ENTRY (gde->_priv->date_entry), buffer);
+	buffer[sizeof(buffer)-1] = '\0';
+
+	str_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+	gtk_entry_set_text (GTK_ENTRY (gde->_priv->date_entry), str_utf8 ? str_utf8 : "");
+	g_free (str_utf8);
 
 	/* Set the time */
 	if (gde->_priv->flags & GNOME_DATE_EDIT_24_HR) {
@@ -652,7 +673,11 @@ gnome_date_edit_set_time (GnomeDateEdit *gde, time_t the_time)
 		if (strftime (buffer, sizeof (buffer), "%I:%M %p", mytm) == 0)
 			strcpy (buffer, "???");
 	}
-	gtk_entry_set_text (GTK_ENTRY (gde->_priv->time_entry), buffer);
+	buffer[sizeof(buffer)-1] = '\0';
+
+	str_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+	gtk_entry_set_text (GTK_ENTRY (gde->_priv->time_entry), str_utf8 ? str_utf8 : "");
+	g_free (str_utf8);
 }
 
 /**
@@ -707,7 +732,7 @@ create_children (GnomeDateEdit *gde)
 		gtk_widget_show (gde->_priv->cal_label);
 
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-	gtk_box_pack_start (GTK_BOX (hbox), arrow, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), arrow, TRUE, FALSE, 0);
 	gtk_widget_show (arrow);
 
 	gtk_widget_show (gde->_priv->date_button);
@@ -890,8 +915,9 @@ gnome_date_edit_get_time (GnomeDateEdit *gde)
 			}
 		}
 
-		if (flags != NULL) {
+		if (flags != NULL && tm.tm_hour < 12) {
 			char buf[256] = "";
+			char *str_utf8;
 			struct tm pmtm = {0};
 
 			/* Get locale specific "PM", note that it
@@ -899,10 +925,16 @@ gnome_date_edit_get_time (GnomeDateEdit *gde)
 			pmtm.tm_hour = 17; /* around tea time is always PM */
 			if (strftime (buf, sizeof (buf), "%p", &pmtm) == 0)
 				strcpy (buf, "");
+			buf[sizeof(buf)-1] = '\0';
 
-			if (g_strcasecmp (flags, buf) == 0 &&
-			    tm.tm_hour < 12)
-					tm.tm_hour += 12;
+			str_utf8 = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+
+			/* eek, this may be evil, we are sort of fuzzy here */
+			if ((str_utf8 != NULL && strcmp (flags, str_utf8) == 0) ||
+			    g_ascii_strcasecmp (flags, buf) == 0)
+				tm.tm_hour += 12;
+
+			g_free (str_utf8);
 		}
 
 		g_free (string);
