@@ -236,7 +236,10 @@ static void
 icon_get_height (Icon *icon, int *icon_height, int *text_height)
 {
 	double d_icon_height, dy1, dy2;
-	gtk_object_get(GTK_OBJECT(icon->image), "height", &d_icon_height, NULL);
+	if (icon->image != NULL)
+		g_object_get (G_OBJECT (icon->image), "height", &d_icon_height, NULL);
+	else
+		d_icon_height = 0.0;
 	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (icon->text), NULL, &dy1, NULL, &dy2);
 	*icon_height = d_icon_height;
 	*text_height = dy2 - dy1;
@@ -284,26 +287,29 @@ gil_place_icon (Gil *gil, Icon *icon, int x, int y, int icon_height)
 
 	priv = gil->_priv;
 
-	gtk_object_get(GTK_OBJECT(icon->image), "height", &d_icon_image_height, NULL);
-	icon_image_height = d_icon_image_height;
-	g_assert(icon_image_height != 0);
-	if (icon_height > icon_image_height)
-		y_offset = (icon_height - icon_image_height) / 2;
-	else
-		y_offset = 0;
+	if (icon->image != NULL) {
+		g_object_get (G_OBJECT (icon->image), "height", &d_icon_image_height, NULL);
+		icon_image_height = d_icon_image_height;
+		g_assert(icon_image_height != 0);
+		if (icon_height > icon_image_height)
+			y_offset = (icon_height - icon_image_height) / 2;
+		else
+			y_offset = 0;
 
-	gtk_object_get(GTK_OBJECT(icon->image), "width", &d_icon_image_width, NULL);
-	icon_image_width = d_icon_image_width;
-	g_assert(icon_image_width != 0);
-	if (priv->icon_width > icon_image_width)
-		x_offset = (priv->icon_width - icon_image_width) / 2;
-	else
-		x_offset = 0;
+		g_object_get (G_OBJECT (icon->image), "width", &d_icon_image_width, NULL);
+		icon_image_width = d_icon_image_width;
+		g_assert(icon_image_width != 0);
+		if (priv->icon_width > icon_image_width)
+			x_offset = (priv->icon_width - icon_image_width) / 2;
+		else
+			x_offset = 0;
+
+		gnome_canvas_item_set (GNOME_CANVAS_ITEM (icon->image),
+				       "x",  (double) (x + x_offset),
+				       "y",  (double) (y + y_offset),
+				       NULL);
+	}
 	
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (icon->image),
-			       "x",  (double) (x + x_offset),
-			       "y",  (double) (y + y_offset),
-			       NULL);
 
 	gnome_icon_text_item_setxy (icon->text,
 				    x,
@@ -621,7 +627,7 @@ selection_one_icon_event (Gil *gil, Icon *icon, int idx, int on_text, GdkEvent *
 	 * destroyed by one of the signal handlers.
 	 */
 	text = icon->text;
-	gtk_object_ref (GTK_OBJECT (text));
+	g_object_ref (G_OBJECT (text));
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
@@ -669,7 +675,7 @@ selection_one_icon_event (Gil *gil, Icon *icon, int idx, int on_text, GdkEvent *
 	if (on_text && retval)
 		gtk_signal_emit_stop_by_name (GTK_OBJECT (text), "event");
 
-	gtk_object_unref (GTK_OBJECT (text));
+	g_object_unref (G_OBJECT (text));
 
 	return retval;
 }
@@ -759,7 +765,7 @@ selection_many_icon_event (Gil *gil, Icon *icon, int idx, int on_text, GdkEvent
 	 * destroyed by one of the signal handlers.
 	 */
 	text = icon->text;
-	gtk_object_ref (GTK_OBJECT (text));
+	g_object_ref (G_OBJECT (text));
 
 	range = (event->button.state & GDK_SHIFT_MASK) != 0;
 	additive = (event->button.state & GDK_CONTROL_MASK) != 0;
@@ -841,7 +847,7 @@ selection_many_icon_event (Gil *gil, Icon *icon, int idx, int on_text, GdkEvent
 	if (on_text && retval)
 		gtk_signal_emit_stop_by_name (GTK_OBJECT (text), "event");
 
-	gtk_object_unref (GTK_OBJECT (text));
+	g_object_unref (G_OBJECT (text));
 
 	return retval;
 }
@@ -962,16 +968,20 @@ icon_new_from_pixbuf (GnomeIconList *gil, GdkPixbuf *im,
 
 	icon->icon_filename = g_strdup (icon_filename);
 
-	icon->image = GNOME_CANVAS_PIXBUF (gnome_canvas_item_new (
-		GNOME_CANVAS_GROUP (canvas->root),
-		GNOME_TYPE_CANVAS_PIXBUF,
-		"x", 0.0,
-		"y", 0.0,
-		"width", (double) gdk_pixbuf_get_width (im),
-		"height", (double) gdk_pixbuf_get_height (im),
-		"pixbuf", im,
-		"anchor", GTK_ANCHOR_NW,
-		NULL));
+	if (im != NULL) {
+		icon->image = GNOME_CANVAS_PIXBUF (gnome_canvas_item_new (
+			GNOME_CANVAS_GROUP (canvas->root),
+			GNOME_TYPE_CANVAS_PIXBUF,
+			"x", 0.0,
+			"y", 0.0,
+			"width", (double) gdk_pixbuf_get_width (im),
+			"height", (double) gdk_pixbuf_get_height (im),
+			"pixbuf", im,
+			"anchor", GTK_ANCHOR_NW,
+			NULL));
+	} else {
+		icon->image = NULL;
+	}
 
 	icon->text = GNOME_ICON_TEXT_ITEM (gnome_canvas_item_new (
 		GNOME_CANVAS_GROUP (canvas->root),
@@ -982,27 +992,29 @@ icon_new_from_pixbuf (GnomeIconList *gil, GdkPixbuf *im,
 					0, 0, priv->icon_width, NULL,
 					text, priv->is_editable, priv->static_text);
 	
-	gtk_signal_connect (GTK_OBJECT (icon->image), "event",
-			    GTK_SIGNAL_FUNC (icon_event),
-			    icon);
-	icon->text_event_id = gtk_signal_connect (GTK_OBJECT (icon->text), "event",
-						  GTK_SIGNAL_FUNC (icon_event),
-						  icon);
+	if (icon->image != NULL) {
+		g_signal_connect (G_OBJECT (icon->image), "event",
+				  G_CALLBACK (icon_event),
+				  icon);
+	}
+	icon->text_event_id = g_signal_connect (G_OBJECT (icon->text), "event",
+						G_CALLBACK (icon_event),
+						icon);
 
 #if 0
-	gtk_signal_connect (GTK_OBJECT (icon->text), "editing_started",
-			    GTK_SIGNAL_FUNC (editing_started),
-			    icon);
-	gtk_signal_connect (GTK_OBJECT (icon->text), "editing_stopped",
-			    GTK_SIGNAL_FUNC (editing_stopped),
-			    icon);
+	g_signal_connect (G_OBJECT (icon->text), "editing_started",
+			  G_CALLBACK (editing_started),
+			  icon);
+	g_signal_connect (G_OBJECT (icon->text), "editing_stopped",
+			  G_CALLBACK (editing_stopped),
+			  icon);
 
-	gtk_signal_connect (GTK_OBJECT (icon->text), "text_changed",
-			    GTK_SIGNAL_FUNC (text_changed),
-			    icon);
-	gtk_signal_connect (GTK_OBJECT (icon->text), "height_changed",
-			    GTK_SIGNAL_FUNC (height_changed),
-			    icon);
+	g_signal_connect (G_OBJECT (icon->text), "text_changed",
+			  G_CALLBACK (text_changed),
+			  icon);
+	g_signal_connect (G_OBJECT (icon->text), "height_changed",
+			  G_CALLBACK (height_changed),
+			  icon);
 #endif
 
 	return icon;
@@ -1022,7 +1034,7 @@ icon_new (Gil *gil, const char *icon_filename, const char *text)
 	retval = icon_new_from_pixbuf (gil, im, icon_filename, text);
 
 	if(im)
-		gdk_pixbuf_unref(im);
+		g_object_unref (G_OBJECT (im));
 
 	return retval;
 }
@@ -1193,11 +1205,19 @@ icon_destroy (Icon *icon)
 {
 	if (icon->destroy)
 		(* icon->destroy) (icon->data);
+	icon->destroy = NULL;
+	icon->data = NULL;
 
 	g_free (icon->icon_filename);
+	icon->icon_filename = NULL;
+
+	if (icon->image != NULL)
+		gtk_object_destroy (GTK_OBJECT (icon->image));
+	icon->image = NULL;
 
-	gtk_object_destroy (GTK_OBJECT (icon->image));
 	gtk_object_destroy (GTK_OBJECT (icon->text));
+	icon->text = NULL;
+
 	g_free (icon);
 }
 
@@ -1339,12 +1359,12 @@ gil_destroy (GtkObject *object)
 	}
 
 	if (gil->adj) {
-		gtk_object_unref (GTK_OBJECT (gil->adj));
+		g_object_unref (G_OBJECT (gil->adj));
 		gil->adj = NULL;
 	}
 	
 	if (gil->hadj) {
-		gtk_object_unref (GTK_OBJECT (gil->hadj));
+		g_object_unref (G_OBJECT (gil->hadj));
 		gil->hadj = NULL;
 	}
 	
@@ -1761,7 +1781,7 @@ gil_button_press (GtkWidget *widget, GdkEventButton *event)
 						"width_pixels", 1,
 						"outline_stipple", stipple,
 						NULL);
-	gdk_bitmap_unref (stipple);
+	g_object_unref (G_OBJECT (stipple));
 
 	gnome_canvas_item_grab (priv->sel_rect, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
 				NULL, event->time);
@@ -1780,10 +1800,13 @@ icon_is_in_area (Icon *icon, int x1, int y1, int x2, int y2)
 	if (x1 == x2 && y1 == y2)
 		return FALSE;
 
-	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (icon->image), &ix1, &iy1, &ix2, &iy2);
+	if (icon->image != NULL) {
+		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (icon->image),
+					      &ix1, &iy1, &ix2, &iy2);
 
-	if (ix1 <= x2 && iy1 <= y2 && ix2 >= x1 && iy2 >= y1)
-		return TRUE;
+		if (ix1 <= x2 && iy1 <= y2 && ix2 >= x1 && iy2 >= y1)
+			return TRUE;
+	}
 
 	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (icon->text), &ix1, &iy1, &ix2, &iy2);
 
@@ -2288,12 +2311,12 @@ gnome_icon_list_set_hadjustment (GnomeIconList *gil, GtkAdjustment *hadj)
 	old_adjustment = gil->hadj;
 
 	if (gil->hadj)
-		gtk_object_unref (GTK_OBJECT (gil->hadj));
+		g_object_unref (G_OBJECT (gil->hadj));
 
 	gil->hadj = hadj;
 
 	if (gil->hadj) {
-		gtk_object_ref (GTK_OBJECT (gil->hadj));
+		g_object_ref (G_OBJECT (gil->hadj));
 		/* The horizontal adjustment is not used, so set some default
 		 * values to indicate that everything is visible horizontally.
 		 */
@@ -2340,18 +2363,18 @@ gnome_icon_list_set_vadjustment (GnomeIconList *gil, GtkAdjustment *vadj)
 
 	if (gil->adj) {
 		gtk_signal_disconnect_by_data (GTK_OBJECT (gil->adj), gil);
-		gtk_object_unref (GTK_OBJECT (gil->adj));
+		g_object_unref (G_OBJECT (gil->adj));
 	}
 
 	gil->adj = vadj;
 
 	if (gil->adj) {
-		gtk_object_ref (GTK_OBJECT (gil->adj));
+		g_object_ref (G_OBJECT (gil->adj));
 		gtk_object_sink (GTK_OBJECT (gil->adj));
-		gtk_signal_connect (GTK_OBJECT (gil->adj), "value_changed",
-				    GTK_SIGNAL_FUNC (gil_adj_value_changed), gil);
-		gtk_signal_connect (GTK_OBJECT (gil->adj), "changed",
-				    GTK_SIGNAL_FUNC (gil_adj_value_changed), gil);
+		g_signal_connect (G_OBJECT (gil->adj), "value_changed",
+				  G_CALLBACK (gil_adj_value_changed), gil);
+		g_signal_connect (G_OBJECT (gil->adj), "changed",
+				  G_CALLBACK (gil_adj_value_changed), gil);
 	}
 
 	if (!gil->adj || !old_adjustment)
@@ -2413,7 +2436,7 @@ gnome_icon_list_new (guint icon_width, GtkAdjustment *adj, int flags)
 {
 	Gil *gil;
 
-	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+	gtk_widget_push_colormap (gdk_rgb_get_colormap ());
 	gil = GIL (gtk_type_new (GNOME_TYPE_ICON_LIST));
 	gtk_widget_pop_colormap ();
 
@@ -2854,10 +2877,12 @@ gnome_icon_list_get_icon_at (GnomeIconList *gil, int x, int y)
 
 	for (n = 0; n < priv->icon_list->len; n++) {
 		Icon *icon = g_array_index(priv->icon_list, Icon*, n);
-		GnomeCanvasItem *image = GNOME_CANVAS_ITEM (icon->image);
+		/* Note: this isn't the checking cast, because icon->image
+		 * could be NULL */
+		GnomeCanvasItem *image = (GnomeCanvasItem *) (icon->image);
 		GnomeCanvasItem *text = GNOME_CANVAS_ITEM (icon->text);
 
-		if (wx >= image->x1 && wx <= image->x2 && wy >= image->y1 && wy <= image->y2) {
+		if (image != NULL && wx >= image->x1 && wx <= image->x2 && wy >= image->y1 && wy <= image->y2) {
 			dist = (* GNOME_CANVAS_ITEM_GET_CLASS (image)->point) (
 				image,
 				wx, wy,
